//+------------------------------------------------------------------+
//|                                       jtSmoothTrendMultTF_0_1.mq4 |
//|                                      Copyright 2022, John Turner |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2022, John Turner"
#property link      "https://www.mql5.com"
#property version   "1.00"
#property strict
#property indicator_separate_window
//#property indicator_minimum -5
//#property indicator_maximum 5
#property indicator_buffers 12
#property indicator_plots   6

//--- measure trendTF1
#property indicator_label1  "trendTF1"
#property indicator_type1   DRAW_NONE
#property indicator_color1  clrOrangeRed
#property indicator_style1  STYLE_SOLID
#property indicator_width1  3
//--- measure trendTF2
#property indicator_label2  "trendTF2"
#property indicator_type2   DRAW_NONE
#property indicator_color2  clrWhiteSmoke
#property indicator_style2  STYLE_SOLID
#property indicator_width2  1
//--- measure trendTF3
#property indicator_label3  "trendTF3"
#property indicator_type3   DRAW_NONE
#property indicator_color3  clrOrangeRed
#property indicator_style3  STYLE_SOLID
#property indicator_width3  3
//--- measure trendTF4
#property indicator_label4  "trendTF4"
#property indicator_type4   DRAW_NONE
#property indicator_color4  clrWhiteSmoke
#property indicator_style4  STYLE_SOLID
#property indicator_width4  1
//--- plot dirMult
#property indicator_label5  "dirMult"
#property indicator_type5   DRAW_HISTOGRAM
#property indicator_color5  clrOrangeRed
#property indicator_style5  STYLE_SOLID
#property indicator_width5  3
//--- plot dirLong
#property indicator_label6  "slopeScaled"
#property indicator_type6   DRAW_LINE
#property indicator_color6  clrWhiteSmoke
#property indicator_style6  STYLE_SOLID
#property indicator_width6  3

#include <jtCommonFunctions.mqh>

//enum trend
//{
//   upTrend,
//   downTrend,
//   noTrend
//};

//--- indicator buffers
double         trendTF1[];
double         trendTF2[];
double         trendTF3[];
double         trendTF4[];
double         dirMult[];     // 0 = down, 1 = up
double         slopeScaled[];
double         trendTF4Raw[];
double         trendScaled[];
double         normMean[];
double         normStdDev[];
double         tempArray[];

//--- variables
input ENUM_TIMEFRAMES TimeFrame1 = 60; // Lowest Time Frame
input ENUM_TIMEFRAMES TimeFrame2 = 240;
input ENUM_TIMEFRAMES TimeFrame3 = 240;
input ENUM_TIMEFRAMES TimeFrame4 = 1440;  // Highest Time Frame
input double TF1_Period = 25;
input double TF2_Period = 25;
input double TF3_Period = 25;
input double TF4_Period = 10;
input ENUM_MA_METHOD TF1_MA_Method = 1; //0 = Simple MA, 1 = Exponential MA, 2 = Smoothed MA, 3 = Linear-weighted MA
input ENUM_MA_METHOD TF2_MA_Method = 1;
input ENUM_MA_METHOD TF3_MA_Method = 2;
input ENUM_MA_METHOD TF4_MA_Method = 1;
input double run = 28;  // Number of TF4 bars to use for slope calculation
//input double scalingSet = 200;   // Number of Bars to scale slope against
bool upTrend;
bool downTrend;
//input double          upOneThresh = 0.1;
//input double          upTwoThresh = 0.1;

//#define run (7)
#define scalingSet (2000)

int run1;
//int run2;

//trend dir1;
//trend dir2;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {
//--- indicator buffers mapping
   SetIndexBuffer(0,trendTF1);
   SetIndexBuffer(1,trendTF2);
   SetIndexBuffer(2,trendTF3);
   SetIndexBuffer(3,trendTF4);
   SetIndexBuffer(4,dirMult);
   SetIndexBuffer(5,slopeScaled);
   SetIndexBuffer(6,trendTF4Raw);
   SetIndexBuffer(7,trendScaled);
   SetIndexBuffer(8,normMean);
   SetIndexBuffer(9,normStdDev);
   SetIndexBuffer(10,tempArray);
   
   ArraySetAsSeries(trendTF1,true);
   ArraySetAsSeries(trendTF2,true);
   ArraySetAsSeries(trendTF3,true);
   ArraySetAsSeries(trendTF4,true);
   ArraySetAsSeries(dirMult,true);     
   ArraySetAsSeries(slopeScaled,true);
   ArraySetAsSeries(trendTF4Raw,true);
   ArraySetAsSeries(trendScaled,true);
   ArraySetAsSeries(normMean,true);
   ArraySetAsSeries(normStdDev,true);
   ArraySetAsSeries(tempArray,true);
   
   ObjectsDeleteAll();
   
   upTrend = False;
   downTrend = False;
   
   //run1 = run * (TimeFrame4 / TimeFrame3);
   run1 = run;
   //run2 = run * (upTwoTimeFrame / upOneTimeFrame);
   
//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
//---
   int i;
   static int limit;
   double temp;
   //double temp2;
   //int j;
   double rise;
   double tempSlope;
   //double NormalizationSet = 200;   // Normalize the slope calculation to the standard deviation of this many bars
   //double min;
   //double max;

   if(rates_total <= run)
      return 0; 
        
   //--- last counted bar will be recounted
   limit=rates_total-prev_calculated;
   if(prev_calculated>0)
      limit++;     
    
   for(i=0; i<limit; i++)   
   {
      trendTF1[i] = iCustom(NULL, 0, "Super smoothed average trend 1.2", TimeFrame1, TF1_Period, 0, TF1_MA_Method, false, false, true, true, false, false, false, clrLimeGreen, clrOrangeRed, clrDimGray, 3, true, 2, i); // Mode 2 indicates direction 
      trendTF2[i] = iCustom(NULL, 0, "Super smoothed average trend 1.2", TimeFrame2, TF2_Period, 0, TF2_MA_Method, false, false, true, true, false, false, false, clrLimeGreen, clrOrangeRed, clrDimGray, 3, true, 2, i); 
      trendTF3[i] = iCustom(NULL, 0, "Super smoothed average trend 1.2", TimeFrame3, TF3_Period, 0, TF3_MA_Method, false, false, true, true, false, false, false, clrLimeGreen, clrOrangeRed, clrDimGray, 3, true, 2, i); 
      trendTF4[i] = iCustom(NULL, 0, "Super smoothed average trend 1.2", TimeFrame4, TF4_Period, 0, TF4_MA_Method, false, false, true, true, false, false, false, clrLimeGreen, clrOrangeRed, clrDimGray, 3, true, 2, i); 
      trendTF4Raw[i] = iCustom(NULL, 0, "Super smoothed average trend 1.2", TimeFrame4, TF4_Period, 0, TF4_MA_Method, false, false, true, true, false, false, false, clrLimeGreen, clrOrangeRed, clrDimGray, 3, true, 1, i); // Mode 1 is raw value

      //if((upTrend == false) && (downTrend == false))
      //{
         if(trendTF1[i] == 2147483647) // UP
         {
            if(trendTF2[i] == 2147483647) // UP
            {
               if(trendTF3[i] == 2147483647) // UP
               {
                  upTrend = true;
                  dirMult[i] = 1;
               }
            }
         }
         else if(trendTF1[i] != 2147483647) // DOWN
         {
            if(trendTF2[i] != 2147483647) // DOWN
            {
               if(trendTF3[i] != 2147483647) // DOWN
               {
                  downTrend = true;
                  dirMult[i] = -1;
               }
            }
         }
      //}
   }


   /*  --- KEEP THIS CODE FOR FINDING STANDARD DEVIATION ---
   for(i=0; i<limit; i++)
   {            
      if(NormalizationSet < rates_total)     // Normalize the data set to the standard deviation of the data set so indicator will work on any chart
      {
         temp = 0;
         for(j=0; j<NormalizationSet; j++)
         {
            if((i+j) < rates_total)
               temp = temp + trendTF4Raw[i+j]; 
            else temp = 0;       
         }
         normMean[i] = temp / NormalizationSet;   // Calculate the mean of the normalization set
        
         temp = 0;
         temp2 = 0;
         for(j=0; j<NormalizationSet; j++)
         {
            if((i+j) < rates_total)
            {
               temp2 = (trendTF4Raw[i+j] - normMean[i]) * (trendTF4Raw[i+j] - normMean[i]);
               temp = temp + temp2;  // Calculate the sum of the squares of the difference from the mean
            }
            else
            {
               temp2 = 1;
               temp = 1;
            }
         }
         temp2 = temp / NormalizationSet;
         normStdDev[i] = sqrt(temp2);   // Calculate standard deviation
      }
      else
      {
         normMean[i] = 1;
         normStdDev[i] = 1;
      }
      
      trendNormalized[i] = (trendTF4Raw[i] - normMean[i]) / normStdDev[i];  // Calculate z-score normalization and put in normalized array
   }
   */  
    
   ArrayCopy(tempArray, trendTF4Raw, 0, 0, WHOLE_ARRAY); 
      
   for(i=0; i<limit; i++)
   {
      int Q1;
      int Q3;
      double IQR; // Interquartile range
      double median;
      static int barsCalculated = 0;

      //ArrayCopy(tempArray, trendTF4Raw, i, i, WHOLE_ARRAY);
      ArraySort(tempArray, scalingSet, i, MODE_ASCEND);
      Q3 = i + MathRound(scalingSet * 0.75);
      Q1 = i + MathRound(scalingSet * 0.25);
      IQR = tempArray[Q3] - tempArray[Q1];
      median = tempArray[Q1 + ((Q3 - Q1)/2)];
      
      trendScaled[i] = (trendTF4Raw[i] - median) / IQR;  // Robust scaling method    
      
      //if(rates_total <= run1)
      //{
      ////   //slopeScaled[i] = (trendScaled[i] - trendScaled[i+run1]) / run1;
      //   //slopeScaled[i] = trendScaled[i]*5;
      //   slopeScaled[i] = -2;
      ////   //slopeScaled[i] = 2;
      //}
      //else
      //{
      ////   slopeScaled[i] = -2;
      //   slopeScaled[i] = trendScaled[i]*3;
      //}
      
      //if(barsCalculated <= run1)
      //   barsCalculated++;
      slopeScaled[i] = 3;
   }
 
   
   for(i=0; i<limit; i++)
   {
      //if(rates_total <= run1)
      //{
      //   slopeScaled[i] = -2;
      //}
      //else
      //{
      //   slopeScaled[i] = trendScaled[i]*3;
      //}
      
      
      
      
   
      //if((i+run1) < rates_total)
      //{
      //   //slopeScaled[i] = (trendScaled[i] - trendScaled[i+run1]) / run1;
      //   //slopeScaled[i] = trendScaled[i];
      //   slopeScaled[i] = 2;
      //}
      //else
      //{
      //   slopeScaled[i] = -2;
      //}
   
      //if((i+run1) < rates_total)
      //   rise = trendScaled[i] - trendScaled[i+run1];
      //else rise = 1;
    
      //tempSlope = rise / run1; 
      //slopeScaled[i] = tempSlope;
      //slopeScaled[i] = trendScaled[i];
      ///slopeScaled[i] = (rise / run1) * 500;  // Scale slope value to 0 to 5
      
      //if(trendTF4[i] == 2147483647)  //Uptrend
      //{
      //   if(rise < 0) // If in uptrend but downward slope
      //   {
      //      slopeLong[i] = 0;
      //   }
      //   else
      //   {
      //      slopeLong[i] = tempSlope * slopeMultiplier;
      //   }
      //}
      //else   //Downtrend
      //{
      //   if(rise > 0) // If in downtrend but upward slope
      //   {
      //      slopeLong[i] = 0;
      //   }
      //   else
      //   {
      //      slopeLong[i] = tempSlope * slopeMultiplier;
      //   }
      //}

   }
 
   
//--- return value of prev_calculated for next call
   return(rates_total);
  }
//+------------------------------------------------------------------+
//| ChartEvent function                                              |
//+------------------------------------------------------------------+
void OnChartEvent(const int id,
                  const long &lparam,
                  const double &dparam,
                  const string &sparam)
  {
//---
   
  }
//+------------------------------------------------------------------+
